
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

/* Program DPROFILE Version 1.20 */
/* Author Larry Curcio */
/* Some code similarities with ROCKET4.BAS by John Kallend. */
/* CP code mostly lifted from Martin Catt, CPRESS.BAS */

/* Version 1.1 has Par surface value added. */

/* Version 1.2 has Fin/Body Interference represented as .002 * Root^2 */
/* (Can Switch to Extended Area by setting Rocket->DoExtended = 1) */


/* IF COMPILING ON TURBO C FOR DOS, SET TURBODOS TO 1 - OTHERWISE 0 */
#define TURBODOS 0

/* Turbo C++ For Windows */

#define TURBOWIN 1

/* IF COMPILING ON ANSI WITH CURSOR CONTROL, SET ANSI TO 1 - OTHERWISE 0 */
/* EXAMPLES OF ANSI CURSOR CONTROL MASCHINES ARE VTXXX WHERE XXX>=100, AND */
/* IBM PC WITH ANSI.SYS INSTALLED AS A DRIVER. (THOUH IBM OPTION BETTER) */

#define ANSI 0

/* CLS is the Screen Clear function. Define as clrscr for TURBO C */
/* define as printf("\n\n\n\n\n") for others */

#define UNIX 0

/*
   IF SENSITIVE IS SET TO 1 *OR* IF UNIX IS SET TO 1, THEN FILE NAMES AND
   DIRECTORY NAMES ARE CASE SENSITIVE. OTHERWISE FILE NAMES ARE CASE
   INSENSITIVE
*/
#define SENSITIVE 0
/*-----------------------------------------------------------------------*/
#if(TURBODOS == 1)
/* #define printf  cprintf */
#include <conio.h>
#define CLS clrscr()

#elif (TURBOWIN == 1)
#define CLS clrscr()

#elif ANSI == 1
#define CLS vmscls()

#elif UNIX == 1
#define CLS UnixClr()

#elif UNIX == 2
#include <curses.h>
#define CLS UnixClr()

#else
#define CLS printf("\n\n\n\n\n\n\n\n\n")
#endif


#if ((UNIX == 0) && (SENSITIVE == 0))
#define UPPERCASE UPPERCASE1
#else
#define UPPERCASE(x) x
#endif


#define KALLEND 2
#define GREGOREK 1

#define INTERFERENCE GREGOREK



/* INITIAL DEFAULT TEMPERATURE IN "TEMPUNITS" */
#define TEMPF1 68.0
#define TEMPC1 20.0

/* Size of Drag Profile */
#define DRAG_SIZE 1000

/* Length of Terminal Line */
#define LINE_LENGTH 80
#define NUMBER_COMMENTS 5

/* INTERPOLATION CUTOFF defines num. intervals needing no interpolation */
/* That is, it's the cutoff between long and short thrust curves */
/* It all arises from different standards in the modroc world */
#define INTERPOLATION_CUTOFF 100

/* Number of documentation lines in drag profile */
#define NUMDOC 5

/* Number of repetitions in Newton's method backtracking procedure */
#define NEWTREP 75


/* String Length */
#define SLEN 255


/* *** END HACKERS' AREA *** */


/* YOU PROBABLY WON'T WANT TO CHANGE THESE THINGS */
/* Missing or null value */
#define MISSING -9.9999E20

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#define PI 3.14159265358979
#define GENERIC_ERROR -32767.0
#define RCRIT 100000.0

#define SHARP 1
#define BLUNT 0
#define TRAPEZOIDAL 1
#define ELLIPSOIDAL 2
/****************************************************************************/
/* GRAVITATIONAL CONSTANT IN m/sec^2 */

#define G 9.805

/* ** AIR DENSITY AT 0 DEGREES C at sea level DO NOT CHANGE */
#define RHO 1.2929

#define METRIC 1
#define OGIVE 1
#define CONE 2
#define ELLIPSE 3

/* METERS TO FEET CONVERSION */
#define MTOF 3.28084

/* CENTIMETERS TO FEET CONVERSION */
#define CTOF 0.0328084

#define ITOC 2.54

#define ITOF 0.0833333333

/* OUNCES TO GRAMS CONVERSION */
#define OZTOG 28.35

/* INCHES TO CM CONVERSION */
#define INTOCM 2.54

/* POUNDS TO NEWTONS */
#define LBTONEWT 4.45


/***************************************************************************/
/* GUTS OF THE STAGE STRUCTURE */

typedef struct LugStruct{
			float Length;
			float InsideDiameter;
			float OutsideDiameter;
			float Sll;
			float SllWetted;
			float Surface;
			struct LugStruct * Next;
			}LUGTYPE;

typedef struct FinStruct{
			int NumFins; /* Number of fins in set */
			int FinType;
			float FromNose;
			float AverageChord;
			float RootChord;
			float TipChord;
			int TipType; /* Squared-off, Sharp, Rounded */
			int DoExtended;
			float Surface;
			int SurfaceType; /* Qualitative surface type */
			float Width; /* VFW in old style */
			float Thickness;
			float Sweep;
			float SweepAngle;
			float SweepCosine;
			float FinArea;
			float BodyDiameter;/* Body Diameter at root */
			float ExtendedArea; /* FinArea + Root * Body Diameter/2 */
			struct FinStruct * Next;
			}FINTYPE;

typedef struct SectionStruct{
		     int SurfaceType;
		     float Surface;
		     float Diameter;
		     float Length;
		     int TransitionType;
		     float TransitionLength;
		     float TransitionTipDiameter;
		     float FromNose;/*@@##@@ CALCULATE */
		     float WettedAreaOfBody;
		     int NumFinSets;
		     FINTYPE * Fins;
		     int NumLugs;
		     LUGTYPE * Lugs;
		     int LastSection;
		     struct SectionStruct * Next;
		     }SECTIONTYPE;


typedef struct StageStruct{
			   int NumSections;
			   SECTIONTYPE * Sections;
			   float TotalLength;
			   float WettedAreaOfBody;
			   float TotalWettedAreaOfBody;
			   float MaximumDiameter;
			   struct StageStruct * Next;
			  }STAGETYPE;

typedef struct
{
int NumStages;
STAGETYPE * Stages;
int SurfaceType;
float Surface;
float TotalLength;
float MaximumDiameter;
float TotalWettedAreaOfBody;
int NoseType;
float NoseLength;
float NoseDiameter;
int SharpOrBluntNose;
float BoatTailLength;
float BaseDiameter;
}CONFIGTYPE;

typedef char Label[12];
Label NoseType[4] = {"", "Ogive", "Cone", "Ellipsoid"};
Label SharpOrBlunt[2] = {"Blunt", "Sharp"};
Label SurfaceType[6] = {"", "Slick", "Smooth", "Matt", "Par", "Unfinished"};
Label FinType[4] = {"", "Trapezoidal", "Elliptical", "Unspecified"};
Label TipType[5]={"", "Square", "Round", "Sharp", "UnSpecified"};
Label DoExtended[3] = {"", "Included", "Ignored"};
Label TempUnitsDollar[2] = {"Centigrade", "Fahrenheit"};
Label InputUnits[2] = {"Inches", "Centimeters"};
char LineOut[80];

int Units = 0;

#define FAHRENHEIT 1

float Velocity, Temperature, Temperature1, TemperatureC, TemperatureK;
float Mach1, MCrit;
int IMach1, IMCrit;
char Title[81];

int TempUnits = FAHRENHEIT;

CONFIGTYPE * Rocket = (CONFIGTYPE *) NULL;

float GlobalFinCd, GlobalLugCd, GlobalNoseBodyCd, GlobalBaseCd;
float GlobalXSection, GlobalKVisc, GlobalCP;

/***************************************************************************/
/*  Global Constants */
 char  InUnits, AltUnits, EngDir[SLEN], DrgDir[SLEN];
 float Tmp1,      Alt0 = 0,      DeltaT0,  DeltaT1;
 int   Const_Rho, RepNum;
/***************************************************************************/
#if (ANSI == 1)
void vmscls()
{

#define ESC 27
char try[13]= {ESC,'[','2','H',ESC,'[','2','J',ESC,'[','2','H','\0'};

printf("%s", try);
return;
}
#endif
/***************************************************************************/
#if UNIX == 1
void UnixClr()
{
system("clear");
}
#endif
/***************************************************************************/
#if UNIX == 2
void UnixClr()
{
(void) initscr();
clear();
refresh();
endwin();
}
#endif
/***************************************************************************/
void color_routine()
/*COMPILE OPTION */
/* *** CAN INSERT ROUTINE HERE TO SET BACKGROUND COLOR *** */
{
/* NOTE
    Insert your own color routine here.
 */
/*
#if (TURBODOS == 1)
  textmode(C80);
  textcolor(WHITE);
  textbackground(BLUE);
#endif
*/
return;
}
/***************************************************************************/
int strtrm( char line[] )
{
  char *c1, *c2;
  int i,j,k;

  /* trim whitespace from beginning of string */
  i = strlen(line);
  j = strspn(line, " \t\v\f\a\b\n\r" );
  if( j > 0 )
    {
     c1 = line;
     c2 = c1 + j;
     k = i-j+1;
     memmove( c1, c2, k );
    }

  /* trim trailing whitespace */
  j = strlen(line);
  if( j > 0 )
    {
     while( isspace( line[j-1] ) ) j = j - 1;
     line[j] = '\0';
    }
  return j;
}

/***************************************************************************/

void UPPERCASE1( char *line )
{
  while ( *line != '\0' )
    {
     *line = toupper( *line );
     line++;
    }

  return;
}

/***************************************************************************/
int get_input( char * input, int input_max )
/* to allow room for a "\n", "input_max" must be at least 2 */
{
  int i, c;
  c = getc( stdin );
  if( (c != '\n') && (c != EOF) )
    {
     ungetc( c, stdin );
     fgets( input, input_max, stdin );
     i = strtrm( input );    /* drop the "\n" and get a character count */
     UPPERCASE1( input );
    }
  else
    {
     strcpy( input, "") ;
     i = 0;
    }
  return i;
}
/***************************************************************************/
int editnum(char * instring)
/* *** EDIT FOR VALID NUMERIC *** */
{
int  numpt=0, numblank=0, numneg=0, notblank=0;

while((*instring !='\0') &&(*instring != '\r'))
  {
  if((*instring < '0') || (*instring >'9'))
    {

    if(*instring == ' ')
      {
      if(notblank) ++numblank;
      else numblank=1;
      }
    else  /* (*instring == ' ') */
    if(*instring == '-')
      {
      ++numneg;
      if(notblank)return(FALSE);
      }
    else if(*instring == '.')++numpt;

    else return(FALSE); /* non-numeric character found */

    notblank=1;
    }
  else notblank=1;
  ++instring;
  }
  if(numblank > 2)return(FALSE);
  if(numpt > 1)   return(FALSE);
  if(numneg > 1)  return(FALSE);
  return(TRUE);
}
/***************************************************************************/
float get_val(char * cp)

/* *** ACCEPT NUMERIC FROM KEYBOARD, EDIT FOR VALIDITY, AND CONVERT *** */
{
  float res;

  if(!editnum(cp))return((float)MISSING);
  sscanf(cp, "%f", &res);
  return(res);
}

/***************************************************************************/
void get_line(int upcase,char * cp)

/* *** ACCEPT A LINE FORM THE KEYBOARD *** */
{
  int c;

  for (;;)
    {
    c = getchar();
    if ((c >= 'a') && (c <= 'z') && (upcase))
	c = c + 'A' - 'a';
    if (c == EOF || c == '\n')
      {
      *cp = 0;
      return;
      }
    *cp++ = c;
    }
}

/***************************************************************************/
void PrintCenter(FILE * OutUnit, char * OutString)
{

int i,j;
char Pad[LINE_LENGTH];

j = ((int) LINE_LENGTH - strlen(OutString)) / 2;

for(i=0; i < j; i++)Pad[i] = ' ';
Pad[j] = '\0';
fprintf(OutUnit, "%s%s\n", Pad, OutString);
return;
}
/***************************************************************************/
float prompt(char * instring, float x)
/* *** PROMPT USER FOR NUMERIC VALUE AND ACCEPT *** */
/* *** NUMERIC VALUE IS FLOATING POINT, BUT MORE VERSATILE WITH CASTS *** */
{
int go_on;
float y;
char response[SLEN];

go_on=TRUE;

while (go_on)
  {
  y=x;
  printf("%s (%9.4f) ",instring, x);
  get_line((int) TRUE, response);
  if( response[0] != '\0') y = get_val(response);
  if (y > MISSING)
#if(TURBODOS == 0)
     {
     go_on=FALSE;
     }
#else
    {
    go_on=FALSE;
    gotoxy(1,wherey()-1);
    clreol();
    printf("%s (%9.4f) \n\r",instring, y);
    }
  else
    {
    gotoxy(1,wherey()-1);
    clreol();
    }
#endif
  }

return(y);
}
/***************************************************************************/
char get_reply(legend)

/* *** ACCEPT SINGLE CHARACTER REPLY AND PUT INTO UPPER CASE *** */

char legend[];
{
char reply;
char linein[10];
printf("%s ", legend);

do{
  get_line(1, linein);
  reply = linein[0];
  }while (strlen(linein) != 1);

#if(TURBODOS == 1)
gotoxy(1,wherey()-1);
printf("%s %c \r", legend, reply);
#endif

return(reply);
}
/***************************************************************************/

float get_mach1(float temp)
/* *** FIND MACH 1 FROM TEMPERATURE *** */
{
#define MACH1AT0 331.45
return((float)MACH1AT0*sqrt(temp/273.15));
}
/***************************************************************************/
float GetSpike(float MachNum, float LastCd, int NoseSw)
{
float Spike;

  if(NoseSw == SHARP)
    {
	/* Sharp Nosed Vehicles */
	 if (MachNum < 1.05)
	 {
	   /* (0.9 =< M =< 1.05)  */
#if(MCRIT == 9)
	     Spike= LastCd*(1.0 + 35.5*(MachNum-0.9)*(MachNum-0.9))
#else
	     Spike = LastCd*(1.0 + 12.78*(MachNum-0.8)*(MachNum-0.8));
#endif
	 }
	 else
	 /*  1.05 =< M =< 2.0  */
	 Spike=LastCd*(1.27 + 0.53*exp(-5.2*(MachNum-1.05)));
    }
    else

    {
      /* Round-Nosed Vehicles */
      if (MachNum < 1.2)
	 {
	  /* (0.9 =< M =< 1.2) */
#if(MCRIT == 9)
	   Spike = LastCd * (1.0 + 4.88*exp(1.1*log(MachNum-0.9)))
#else
	   Spike = LastCd * (1.0 + 3.556208468*exp(1.1*log(MachNum-0.8)));
#endif
	}
	else
	/* 1.2 =< M =< 2.0 */
	Spike = LastCd * (2.0 + 0.3*exp(-5.75*(MachNum-1.2)));
    }
return(Spike);
}
/***************************************************************************/
void machmaker(float cd[], float temp, int nosesw)
{
/*COMPILE OPTION*/
/*
This routine accounts for drag divergence in the manner described by
Mandell, Caporaso, and Bengen in _TOPICS IN ADVANCED MODEL ROCKETRY_,
first edition. The authors use a critical mach number of 0.9. I have
converted their model to use a critical mach number of 0.8, but I have
given the user a choice of either value.

To choose a critical mach number, #define MCRIT as either 8 (for 0.8) or 9
(for 0.9). Since some c compilers running under UNIX cannot understand a
#if comparison to a decimal constant, I had to use whole numbers instead
of the values themselves.
*/
#define MCRIT 8


float MACH1, MACHPT9, MACHPT8, fwork, fwork1, MachNum;
int IMCRIT, IMACHPT9, IMACHPT8, IMACH1PT05, IMACH1PT2, i, smaller;


MACH1 = get_mach1(temp);
MACHPT9 = 0.9*MACH1;
IMACHPT9 = (int) MACHPT9;
MACHPT8 = 0.8*MACH1;
IMACHPT8 = (int) MACHPT8;

if ((float) IMACHPT9 < MACHPT9) IMACHPT9++;
if ((float) IMACHPT8 < MACHPT8) IMACHPT8++;

IMACH1PT05 = (int) (MACH1*1.05 + .999999);
IMACH1PT2 =  (int) (MACH1*1.2  + .999999);

#if (MCRIT == 9)
  IMCRIT=IMACHPT9;
  if(IMACHPT9 > DRAG_SIZE) return;
  fwork = cd[IMACHPT9-1];
  fwork1=(float) IMACHPT9;
#else
  IMCRIT=IMACHPT8;
  if(IMACHPT8 > DRAG_SIZE) return;
  fwork = cd[IMACHPT8-1];
  fwork1=(float) IMACHPT8;
#endif

if(nosesw == SHARP)
  {
      /* Sharp Nosed Vehicles */

  if(IMACH1PT05 < DRAG_SIZE) smaller=IMACH1PT05;
  else smaller = DRAG_SIZE;

  for (i=IMCRIT; i<smaller; i++)
      {
      MachNum=fwork1/MACH1;
      fwork1 +=1.0;
      /* (0.9 =< M < 1.05)  */
#if(MCRIT == 9)
      cd[i] = fwork*(1.0 + 35.5*(MachNum-0.9)*(MachNum-0.9));
#else
      cd[i] = fwork*(1.0 + 12.78*(MachNum-0.8)*(MachNum-0.8));
#endif

      }
  for (i=smaller; i<DRAG_SIZE; i++)
      {
      MachNum=fwork1/MACH1;
      fwork1 +=1.0;
      /*  1.05 =< M =< 2.0  */
      cd[i]=fwork*(1.27 + 0.53*exp(-5.2*(MachNum-1.05)));
      }
  }
else

  {
	   /* Round-Nosed Vehicles */
  if(IMACH1PT2 < DRAG_SIZE) smaller=IMACH1PT2;
  else smaller = DRAG_SIZE;
for (i=IMCRIT; i<smaller; i++)
      {
      MachNum=fwork1/MACH1;
      fwork1 +=1.0;
      /* (0.9 =< M =< 1.2) */
#if(MCRIT == 9)
      cd[i] = fwork * (1.0 + 4.88*exp(1.1*log(MachNum-0.9)));
#else
      cd[i] = fwork * (1.0 + 3.556208468*exp(1.1*log(MachNum-0.8)));
#endif
      }
  if(smaller < DRAG_SIZE) for (i=smaller; i<DRAG_SIZE
  ; i++)
      {
      MachNum=fwork1/MACH1;
      fwork1 +=1.0;
      /* 1.2 =< M =< 2.0 */
      cd[i] = fwork * (2.0 + 0.3*exp(-5.75*(MachNum-1.2)));
      }
  }
  return;
}
/***************************************************************************/
float sqr(float x)
{
return (x*x);
}
/****************************************************************************/
float GetSurface(int SurfaceType)
{
#define SLICK 1
#define SMOOTH 2
#define MATT 3
#define PAR 4
#define UNFINISHED 5

switch(SurfaceType)
  {
  case SLICK: return(.0001/12.0);
  case SMOOTH: return(.0005/12.0);
  case MATT: return(.001/12.0);

  case PAR: return(.0015/12.0);
  case UNFINISHED: return(.008/12.0);
  default: return(GENERIC_ERROR);
  }
}
/****************************************************************************/
float GetCritK(float Velocity)
{

/*return(.0154/Velocity);*/
return(100 * GlobalKVisc)/Velocity;
}
/****************************************************************************/
float expo(float x, float y)
{
if (x > 0) return(exp(y*log(x)));
else return(GENERIC_ERROR);
}
/****************************************************************************/
float GetRn(float Velocity, float Length)
{
float Rn;

Rn = Velocity*Length/GlobalKVisc;
Rn = Rn - Alt0*0.2666667;
/*Rn = Rn+Alt0*(6300-7900)/6000;*/
if (Rn < 0.0) Rn = 0.001;

return(Rn);
}
/****************************************************************************/
float GetCCrit(float Rn, float Velocity)
{

/*return(Rn/(6100*Velocity));*/
return((Rn +  Alt0 * 0.2666667) * GlobalKVisc / Velocity);
}
/****************************************************************************/
float GetCfBody (float Velocity, float Length, float Surface)
{
float Rn, Cf;

if(Surface > GetCritK(Velocity))
  {
  Cf = .032*expo(Surface/Length, 0.20);
  }
else
 {
 Rn=GetRn(Velocity, Length);
/* Cf = 1/expo(1.5024*log(Rn)-5.6, 0.20);*/
Cf = 1.0/sqr(1.5024*log(Rn)-5.6);

 }

/* Correction for curvature */
Cf += .0016 * (Length/Rocket->MaximumDiameter)/expo(Rn, 0.40);


return(Cf);
}
/****************************************************************************/
float GetNoseBodyCd(float Velocity,
		    float Length,
		    float Diameter,
		    float BaseDiameter,
		    float Surface,
		    float WettedArea
		    )
{
float Cf, Cd, Ratio, SSBody, DOL, DOL3;

/* Do this Before Units Conversion */

DOL = Diameter / Length;

if(Units == METRIC)
  {
  WettedArea *= CTOF * CTOF;
  Length *= CTOF;
  Diameter *= CTOF;
  BaseDiameter *= CTOF;
  }
else
  {
  WettedArea *= ITOF * ITOF;
  Length *= ITOF;
  Diameter *= ITOF;
  BaseDiameter *= ITOF;
  }

  /* Note that Global Units are in End Units */
SSBody = WettedArea / GlobalXSection;

Ratio = BaseDiameter/Diameter;

  {

  DOL3 = DOL * DOL * DOL;

  Cd = 1.02 * GetCfBody(Velocity, Length, Surface) *
     SSBody*(1.0 + 1.5 * sqrt(DOL3) + 7.0 * DOL3); /* Hoerner 6-17 Eq 28 */
  }

GlobalBaseCd = 0.029*Ratio*Ratio*Ratio/sqrt(Cd);

return(Cd + GlobalBaseCd);

}
/****************************************************************************/
float GetTurbulentCfFins(float Chord, float Surface)
{

return(.032*expo(Surface/Chord,0.2));

}
/****************************************************************************/
float GetTurbulentCfFinsNoSurface(float Rn)
{
return(1/sqr(1.5024*log(Rn)-5.6) -310.0/Rn);
}
/****************************************************************************/
float GetLaminarCfFins(float Velocity, float Chord)
{

return(1.33/sqrt(GetRn(Velocity, Chord)));
}
/****************************************************************************/
float GetFinCd1(float Velocity, float Chord, float Thickness, float Surface)
{
float ToC, ToC2;



ToC = Thickness/Chord;
/* ToC = Thickness/Root;*/ /* Ala Gregorek */
ToC2 = ToC*ToC;


if(Surface > GetCritK(Velocity) )
  {
  float Cd;

  Cd = 2* GetTurbulentCfFins(Chord, Surface) * (1.0 + ToC + ToC
	  + 60 * ToC2 * ToC2 );

  return(Cd);
  }
else
  {
  float Rn, CdL, CdT, CCrit;

  CdL = GetLaminarCfFins(Velocity, Chord) * 2.0 * (1.0 + ToC) + ToC2;

  if((Rn = GetRn(Velocity, Chord)) <=  RCRIT)return(CdL);

  CCrit = GetCCrit(RCRIT, Velocity);

  CdT = 2* GetTurbulentCfFinsNoSurface(Rn) * (1.0 + ToC + ToC
	   + 60 * ToC2 * ToC2 );

  /* return(Factor * CdT);  */
  return((CCrit*CdL + (Chord-CCrit)*CdT)/Chord);
  }
}
/****************************************************************************/
float GetFinCd(float Velocity, float Chord, float Thickness,
	       float Surface, float FinWidth,
	       float NFins, float Diameter, float Root,
	       float Tip, int TipType, int DoExtended, int FinType)
{
float CCrit, Cd1, Chord1, Shorter, Longer, TipFraction, TipEffect;
float ExtendedArea, Interference=0.0, Factor, ToC, ToC2;

if(Units == METRIC)
  {
  Chord = CTOF * Chord;
  Thickness = CTOF * Thickness;
  FinWidth = CTOF * FinWidth;
  Diameter = CTOF * Diameter;
  Root = CTOF * Root;
  Tip = CTOF * Tip;
  }
else
  {
  Chord = ITOF * Chord;
  Thickness = ITOF * Thickness;
  FinWidth = ITOF * FinWidth;
  Diameter = ITOF * Diameter;
  Root = ITOF * Root;
  Tip = ITOF * Tip;
  }

TipFraction = Tip /Chord;
ToC = Thickness / Chord;
ToC2 = ToC * ToC;


switch(TipType)
   {
    case 1: /* Squared-Off Tips */
	    TipEffect = 0.055 * ToC2 * TipFraction;
	    break;

    case 2: /* Rounded Tips */
	    TipEffect = -.002 * ToC2 * TipFraction;
	    break;

    case 3: /* Sharp Tips */
	    TipEffect = -0.0015 * ToC2 * TipFraction;
	    break;

    case 4: /* Unspecified - used for construction of fins from parts */
	    TipEffect = 0.0;
	    break;

    default:/* Something's Wrong! */
	    printf("Bad Tip Type! %i \n", TipType);
	    exit(-99);

   }


/* Fin Area Extended for Interference Drag */
if(DoExtended == 1)
  {
#if INTERFERENCE == GREGOREK
  ExtendedArea = NFins * (Chord * FinWidth  + Diameter * Root * 0.5);
  Interference = 0.0;
#else
  /* Kallend's Model */
  ExtendedArea = NFins * Chord * FinWidth;
  Interference = .002 * (float) NFins * Root*Root / GlobalXSection;
#endif
  }
else
  {
  /* Extended Area Not Used - For Doing Fins Piecewise */
  ExtendedArea = NFins * Chord * FinWidth;
  Interference = 0.0;
  }


/* Factor is area ratio (SS)*/
Factor = ExtendedArea/GlobalXSection;

if(Root > Tip)
  {
  Longer = Root;
  Shorter = Tip;
  }
else
  {
  Longer = Tip;
  Shorter = Root;
  }


if ((FinType != TRAPEZOIDAL) || (Surface > GetCritK(Velocity) ) ||
   (((CCrit = GetCCrit(RCRIT, Velocity)) > Longer)) || (CCrit < Shorter))

   {
   /* Surface Effect Tripped or all laminar, or all partially turbulent */
		 /* Computations Straightforward */

   Cd1 =(GetFinCd1(Velocity,  Chord,  Thickness, Surface));

   }
  else
   {
   float Area1, Area2, FinWidth1;
   Chord1 = (Root + CCrit) * 0.5;
   FinWidth1 = FinWidth * (Root - CCrit)/(Root - Tip);
   Area1 = Chord1 * FinWidth1;
   Cd1 = GetFinCd1(Velocity,  Chord1,  Thickness, Surface) * Area1;

   Chord1 = (Tip + CCrit) * 0.5;/* For Ellipse?*/
   FinWidth1 = FinWidth - FinWidth1;
   Area2 = FinWidth1 * Chord1;
   Cd1 += GetFinCd1(Velocity,  Chord1,  Thickness, Surface) * Area2;
   Cd1 /= (Area1 + Area2);
   }

return(Factor * (Cd1 + TipEffect * TipFraction) + Interference);

}
/****************************************************************************/
void InputFins(FINTYPE * Fins)
{
float x;
int GoOn;

printf("\n");

while((x = prompt("Distance From Nose Tip: ", Fins->FromNose)) <= 0);
Fins->FromNose = x;

while((x = prompt("Number of Fins: ", (float) Fins->NumFins )) <= 0);
Fins->NumFins = (int) x;


while(((x = prompt("Fin Type (1=Trapezoidal 2=Elliptical 3=Unspecified): ",
(float) Fins->FinType)) != 1.0) && (x != 2.0) && (x != 3.0));
Fins->FinType = (int) x;

if(Fins->FinType != TRAPEZOIDAL)
  {
  while((x = prompt("Average Chord: ", Fins->AverageChord)) <= 0);
  Fins->AverageChord = x;
  }

while((x = prompt("Span Right Angle: ", Fins->Width)) <= 0);
Fins->Width = x;

while((x = prompt("Root Chord: ", Fins->RootChord)) <= 0);
Fins->RootChord = x;

if(Fins->FinType != ELLIPSOIDAL)
  {
  while((x = prompt("Tip Chord: ", Fins->TipChord)) == MISSING);
  Fins->TipChord = x;
  }
else 
  {
  Fins->TipChord = 0.0;
  }


if (Fins->FinType == TRAPEZOIDAL)
   {
   Fins->AverageChord = 0.5 * (Fins->RootChord + Fins->TipChord);
   }

if(Fins->FinType != ELLIPSOIDAL)
  {
  while((x = prompt("TipType (1=Squared-Off, 2=Rounded, 3=Sharp, 4=No Tip Effect: ", (float) Fins->TipType )) <= 0);
  Fins->TipType = (int) x;
  }
else
  {
  Fins->FinType = 2;
  }

 /*
while((x = prompt("Include Fin/Body Interference? (1=yes, 2=No): ", (float) Fins->DoExtended)) <= 0);
  Fins->DoExtended = (int) x;
*/

for(GoOn = TRUE;
    GoOn;
    GoOn=((Fins->Surface=GetSurface(Fins->SurfaceType)) == GENERIC_ERROR))
    {
    while((x = prompt("Surface(1=Slick, 2=Smooth, 3=Matt, 4=Par, 5=Unfinished:", (float) Fins->SurfaceType )) <= 0);
    Fins->SurfaceType = (int) x;
    }

while((x = prompt("Fin Thickness: ", Fins->Thickness)) <= 0);
Fins->Thickness = x;

/* This should be extended to enter sweep or angle, or neither if you like */
/*
while((x = prompt("Sweep Angle in Degrees: ", Fins->SweepAngle)) < 0);
Fins->SweepAngle = x;

Fins->SweepCosine = cos((PI+PI) * Fins->SweepAngle/360.0);

Fins->Sweep = Fins->Width * sin((PI+PI) * Fins->SweepAngle/360.0);
*/

while((x = prompt("Sweep Distance: ", Fins->Sweep)) < 0);
Fins->Sweep = x;

Fins->SweepAngle = atan(Fins->Sweep / Fins->Width);


/* End Sweep Section */

Fins->FinArea = Fins->Width * Fins->AverageChord;

Fins->ExtendedArea = Fins->FinArea + Fins->BodyDiameter * Fins->RootChord * 0.5;
}
/****************************************************************************/
void GetLugAreas(LUGTYPE * Lug)
{


Lug->Sll = PI * 0.25 * ( Lug->OutsideDiameter * Lug->OutsideDiameter -
		    Lug->InsideDiameter * Lug->InsideDiameter);

Lug->SllWetted = PI * (Lug->OutsideDiameter + Lug->InsideDiameter) * Lug->Length;

return;
}
/****************************************************************************/
void InputLug(LUGTYPE * Lug)
{
float x;
int GoOn;

printf("\n");

while((x = prompt("Lug Length: ", Lug->Length)) <= 0);
Lug->Length = x;

while((x = prompt("Lug Outside Diameter: ", Lug->OutsideDiameter)) <= 0);
Lug->OutsideDiameter = x;

while((x = prompt("Lug Inside Diameter: ", Lug->InsideDiameter)) <= 0);
Lug->InsideDiameter = x;

GetLugAreas(Lug);

}
/****************************************************************************/
float InputSection(SECTIONTYPE * Section, float LastTipDiameter, int SectionNumber)
{
FINTYPE ** Pointer;
LUGTYPE ** LugPointer;
float x, x1;
int i;

if (LastTipDiameter < 0.0)
   {
   while((x = prompt("Section Diameter: ", Section->Diameter)) <= 0);
	  Section->Diameter = x;
   }
else Section->Diameter = LastTipDiameter;

if((SectionNumber != 0) || (Section->Length == 0.0)) x1 = Section->Length;
else x1 = Section->Length + Rocket->NoseLength;

while((x = prompt("Section Length With Any Transition: ", x1)) <= 0);
       Section->Length = x;

if(SectionNumber != 0)Section->Length = x;
else Section->Length = x - Rocket->NoseLength;

while((x = prompt("Transition Length (0 If None): ", Section->TransitionLength)) < 0);
       Section->TransitionLength = x;

if (Section->TransitionLength != 0.0)
   {
/* Note: This is Diameter of Next Section */
   while((x = prompt("Transition Base Diameter: ", Section->TransitionTipDiameter)) < 0);
	  Section->TransitionTipDiameter = x;
   }
else Section->TransitionTipDiameter = Section->Diameter;



while((x = prompt("Number of Launch Lugs: ", (float) Section->NumLugs )) < 0);
Section->NumLugs = (int) x;

LugPointer = &(Section->Lugs);

for(i=0; i < Section->NumLugs; i++)
   {

   if(*LugPointer == (LUGTYPE *) NULL)
     {
     if((*LugPointer = (LUGTYPE *) malloc(sizeof(LUGTYPE))) == (LUGTYPE *) NULL)
       {
       printf("\n** SEVERE ERROR!! **\nCANNOT ALLOCATE LUG \nABORTING\n");
       exit(-99);
       }
     memset(*LugPointer, 0, sizeof(LUGTYPE));

     (*LugPointer)->Surface = Section->Surface;
     (*LugPointer)->Next = (struct LugStruct *) NULL;
     }
   if (Section->NumLugs > 1)printf("\n\nFor Lug %i of Section %i:\n",
					   i+1, SectionNumber);

   InputLug(*LugPointer);
   LugPointer = (LUGTYPE **) &(*LugPointer)->Next;
   } /* for loop */



while((x = prompt("\nNumber of Fin Sets: ", (float) Section->NumFinSets )) < 0);
Section->NumFinSets = (int) x;

Pointer = &(Section->Fins);

for(i=0; i < Section->NumFinSets; i++)
   {

   if(*Pointer == (FINTYPE *) NULL)
     {
     if((*Pointer = (FINTYPE *) malloc(sizeof(FINTYPE))) == (FINTYPE *) NULL)
       {
       printf("\n** SEVERE ERROR!! **\nCANNOT ALLOCATE FIN SET\nABORTING\n");
       exit(-99);
       }
     memset(*Pointer, 0, sizeof(FINTYPE));
     (*Pointer)->TipType = 2; /* Rounded */
     (*Pointer)->SurfaceType = 2; /* Smooth */
     (*Pointer)->FinType = TRAPEZOIDAL;
     (*Pointer)->DoExtended = 1; /* Represent Interference  As Extended Area */
     (*Pointer)->NumFins = 3;
     (*Pointer)->Next = (struct FinStruct *) NULL;
     }
   printf("\n\nFor Fin Set %i of Section %i:\n",
					   i+1, SectionNumber);

   InputFins(*Pointer);
   Pointer = (FINTYPE **) &(*Pointer)->Next;
   } /* for loop */

return(Section->TransitionTipDiameter);
}
/****************************************************************************/
float InputStage(STAGETYPE * Stage, float LastTipDiameter, int LastStage)
{
int i, LastSection;
float x;
SECTIONTYPE ** Pointer;


while((x = prompt("Number of Sections: ", (float) Stage->NumSections )) <= 0);
Stage->NumSections = (int) x;

Pointer = &(Stage->Sections);

for(i=0; i< Stage->NumSections; i++)
   {

   if(*Pointer == (SECTIONTYPE *) NULL)
     {
     if((*Pointer = (SECTIONTYPE *) malloc(sizeof(SECTIONTYPE))) == (SECTIONTYPE *) NULL)
       {
       printf("\n** SEVERE ERROR!! **\nCANNOT ALLOCATE STAGE\nABORTING\n");
       exit(-99);
       }
     memset(*Pointer, 0, sizeof(SECTIONTYPE));
     (*Pointer)->NumFinSets = 1;
     (*Pointer)->NumLugs = 1;
     (*Pointer)->TransitionType = CONE;
     (*Pointer)->Next = (struct SectionStruct *) NULL;
     (*Pointer)->Fins = (FINTYPE *) NULL;
     }

   if(i == (Stage->NumSections-1))LastSection = TRUE;
   else LastSection = FALSE;

   LastSection *= LastStage;

   if (Stage->NumSections > 1)printf("\n\nFor Section %i:\n", Stage->NumSections - i);
   LastTipDiameter = InputSection(*Pointer, LastTipDiameter, i);
   Pointer = (SECTIONTYPE **) &((*Pointer)->Next);
   } /* for loop */

return(LastTipDiameter);
}
/****************************************************************************/
void InputConfig(CONFIGTYPE * Rocket)
{
int i, LastStage, GoOn;
float x, LastTipDiameter = -1.0;
STAGETYPE ** Pointer;

/*
while((x = prompt("Number of Stages: ", (float) Rocket->NumStages )) <= 0);
Rocket->NumStages = (int) x;
*/

while((x = prompt("Ambient Temperature: ", Temperature1)) <= 0);
  Temperature1 = x;

if(TempUnits == FAHRENHEIT)
  {
  Temperature = Temperature1;
  TemperatureC = (float) 5.0 * (Temperature1 -  (float) 32.0)/ (float)9.0 ;
  }
else
  {
  TemperatureC = Temperature1;
  Temperature = TemperatureC * (float) 9.0/(float)5.0    + (float) 32.0;
  }

TemperatureK = TemperatureC + (float) 273.15;

for(GoOn = TRUE;
    GoOn;
    GoOn=((Rocket->Surface=GetSurface(Rocket->SurfaceType)) == GENERIC_ERROR))
    {
    while((x = prompt("Body Surface (1=Slick, 2=Smooth, 3=Matt, 4=Par, 5=Unfinished: ", (float) Rocket->SurfaceType )) <= 0);
    Rocket->SurfaceType = (int) x;
    }


while((x = prompt("Nose Type 1=Ogive, 2=Cone, 3=Ellipsoid: ", (float) Rocket->NoseType )) <= 0);
Rocket->NoseType = (int) x;

if(Rocket->NoseType != ELLIPSE)
  {
  while((x = prompt("Nose Sharp (=1)  or  blunt(=0) : ",
			     (float) Rocket->SharpOrBluntNose )) != 0 && x != 1);

  Rocket->SharpOrBluntNose = (int) x;
  }
else Rocket->SharpOrBluntNose = BLUNT;

  while((x = prompt("Nose Length: ", Rocket->NoseLength)) <= 0);
  Rocket->NoseLength = x;

while((x = prompt("Nose Diameter: ", Rocket->NoseDiameter)) <= 0);
  Rocket->NoseDiameter = x;


Pointer = &(Rocket->Stages);

LastTipDiameter = Rocket->NoseDiameter;

for(i=0; i< Rocket->NumStages; i++)
   {

   if(*Pointer == (STAGETYPE *) NULL)
     {
     if((*Pointer = (STAGETYPE *) malloc(sizeof(STAGETYPE))) == (STAGETYPE *) NULL)
       {
       printf("\n** SEVERE ERROR!! **\nCANNOT ALLOCATE STAGE\nABORTING\n");
       exit(-99);
       }
     memset(*Pointer, 0, sizeof(STAGETYPE));
     (*Pointer)->Next = (struct StageStruct *) NULL;
     (*Pointer)->Sections = (SECTIONTYPE *) NULL;
     (*Pointer)->NumSections = 1;
     }

   if (Rocket->NumStages > 1)printf("\n\nFor Stage %i:\n",Rocket->NumStages - i);

   if(i == (Rocket->NumStages-1))LastStage = TRUE;
   else LastStage = FALSE;

   LastTipDiameter = InputStage(*Pointer, LastTipDiameter, LastStage);
   Pointer = (STAGETYPE **) &((*Pointer)->Next);
   } /* for loop */
}
/****************************************************************************/
void Banner()
{
char Buffer[80];

CLS;
printf("\n\n\n\n\n\n");
PrintCenter(stdout, "** DPROFILE PROGRAM **");
PrintCenter(stdout, "Version 1.20");
printf("\n");
PrintCenter(stdout,
  "Drag Coefficients, Drag Profiles, and Center of Pressure");
printf("\n");
PrintCenter(stdout, "Author: Larry Curcio");
printf("\n");
PrintCenter(stdout,"Cd Methods from S. Hoerner, G. Gregorek, and W. Bengen");
PrintCenter(stdout,"CP Methods from James and Judith Barrowman");
printf("\n");
PrintCenter(stdout, "Some Procedures Borrowed from John Kallend and Martin Catt");

printf("\n");
PrintCenter(stdout, "26 March 1995");
get_line(FALSE, Buffer);
}
/****************************************************************************/
float GetSectionDiameter(SECTIONTYPE * SectionPointer)
{
int i;
FINTYPE * Pointer;

Pointer = SectionPointer->Fins;
/* Put Body Diameter into Fin Structure */

for (i=0; i < SectionPointer->NumFinSets; i++)
    {
    Pointer->BodyDiameter = SectionPointer->Diameter;
    Pointer = (FINTYPE *) Pointer->Next;
    }

return(SectionPointer->Diameter);
}
/****************************************************************************/
float GetStageDiameter(STAGETYPE * StagePointer)
{
int i;
SECTIONTYPE * Pointer;
float SectionDiameter, StageDiameter = 0;

Pointer = StagePointer->Sections;

for (i=0; i < StagePointer->NumSections; i++)
   {
   if(StageDiameter < (SectionDiameter = GetSectionDiameter(Pointer)) )
     {
     StageDiameter = SectionDiameter;
     }
   Pointer = (SECTIONTYPE *) Pointer->Next;
   }

return(StageDiameter);
}
/****************************************************************************/
float GetRocketDiameter(CONFIGTYPE * Rocket)
{
int i;
STAGETYPE * Pointer;
float StageDiameter, MaximumDiameter = 0;

Pointer = Rocket->Stages;

for (i=0; i < Rocket->NumStages; i++)
   {
   if(MaximumDiameter < (StageDiameter = GetStageDiameter(Pointer)) )
     {
     MaximumDiameter = StageDiameter;
     }
   Pointer = (STAGETYPE *) Pointer->Next;
   }

Rocket->MaximumDiameter = MaximumDiameter;



return(MaximumDiameter);
}
/****************************************************************************/
float GetTotalLengthSection(SECTIONTYPE * SectionPointer, float LengthAbove)
{
SectionPointer->FromNose = LengthAbove + SectionPointer->Length
		      - SectionPointer->TransitionLength;
return(SectionPointer->Length);
}
/****************************************************************************/
float GetTotalLengthStage(STAGETYPE * StagePointer, float LengthAbove)
{
int i;
float TotalLengthStage = 0.0;
SECTIONTYPE * Pointer;

Pointer = StagePointer->Sections;

for(i = 0; i< StagePointer->NumSections; i++)
   {
   TotalLengthStage += GetTotalLengthSection(Pointer, LengthAbove);
   Pointer = Pointer->Next;
   }
return(TotalLengthStage);

}
/****************************************************************************/
float GetTotalLength(CONFIGTYPE * Rocket)
{
int i;
float TotalLength;
STAGETYPE * Pointer;

  TotalLength = Rocket->NoseLength;

Pointer = Rocket->Stages;

for(i = 0; i< Rocket->NumStages; i++)
   {
   TotalLength += GetTotalLengthStage(Pointer, TotalLength);
   Pointer = Pointer->Next;
   }
return(TotalLength);

}
/****************************************************************************/
float GetWettedAreaOfSection(int NoseType, float NoseLength, float TransitionTipDiameter,
		    float Diameter, float Length,
		    float TailLength, float BaseDiameter)
{
/* Ratio of Wetted Area */
/* Nose Tip Diameter is nonzero if it's a transition */

float LnDiameter, LnTipDiameter, BodyLength, Work, SSNose;
float NoseArea, TailArea, BodyArea, XSection;

#define ConeArea(D1, D2, H)  ((0.50 * (D1 + D2) * PI *sqrt(sqr(abs(D1 - D2)) + H * H)))


BodyLength = Length - NoseLength - TailLength;

BodyArea = BodyLength * Diameter * PI;


XSection = PI * Diameter * Diameter / 4.0;

switch(NoseType)
   {
   case OGIVE: NoseArea = 2.67 * XSection * NoseLength / Diameter;
	       break;

   case CONE:  if(NoseLength > 0.0) NoseArea = (float) ConeArea(Diameter, TransitionTipDiameter, NoseLength);
	       else NoseArea = 0.0;
	       break;
   case ELLIPSE: if(NoseLength > 0.0)
		   {
		   LnDiameter = NoseLength / Diameter;
		   SSNose = 1.0+2*LnDiameter/sqrt(1.0+sqr(LnDiameter));
		   Work = sqrt(1.0-sqr(1.0/(2*LnDiameter)));
		   Work = atan(Work/sqrt(1.0-sqr(Work)));
		   SSNose = SSNose*Work;
		   NoseArea = SSNose * XSection;
		   }
		 else NoseArea = 0.0;
		 break;
   default: NoseArea = 0.0;

   }

if(TailLength>0) TailArea = (float) ConeArea(Diameter, BaseDiameter, TailLength);
else TailArea = 0.0;


return(NoseArea + TailArea + BodyArea);

}

/****************************************************************************/
float GetStageWettedArea(STAGETYPE * StagePointer)
{
int i;
float StageWettedArea = 0.0;
SECTIONTYPE * Pointer;

for(Pointer = StagePointer->Sections, i=0; i < StagePointer->NumSections;
			   Pointer = (SECTIONTYPE *) Pointer->Next, i++)
    {
    StageWettedArea += GetWettedAreaOfSection(
					      Pointer->TransitionType,
					      0.0,
					      0.0,
					      Pointer->Diameter,
					      Pointer->Length,
					      Pointer->TransitionLength,
					      Pointer->TransitionTipDiameter
					      );
    }


StagePointer->WettedAreaOfBody = StageWettedArea;

return(StageWettedArea);
}
/****************************************************************************/
float GetWettedAreaOfBody(CONFIGTYPE * Rocket)
{
int i;
float WettedAreaOfBody = 0.0;
STAGETYPE * Pointer;

   WettedAreaOfBody = GetWettedAreaOfSection(Rocket->NoseType,
					     Rocket->NoseLength,
					     0.0,
					     Rocket->Stages->Sections->Diameter,
					     Rocket->NoseLength,
					     0.0,
					     0.0
					     );

   for(Pointer = Rocket->Stages, i = 0; i < Rocket->NumStages;
			   Pointer = (STAGETYPE *) Pointer->Next, i++)
      {
      WettedAreaOfBody += GetStageWettedArea(Pointer);
      /* Total Wetted Area Of Body Down to Stage */
      Pointer->TotalWettedAreaOfBody = WettedAreaOfBody;
      }

      Rocket->TotalWettedAreaOfBody = WettedAreaOfBody;

return(WettedAreaOfBody);
}
/****************************************************************************/
float GetLugCd(float Sll ,
	       float SllWetted,
	       float LugLength,
	       float Surface,
	       float Velocity
	       )
{
float CfLug, CdLug;

if(Units == METRIC)
  {
  float CTOF2;

  CTOF2 = CTOF * CTOF;



  SllWetted =   CTOF2* SllWetted;
  Sll = CTOF2 * Sll;

  LugLength = CTOF * LugLength;
  }
else
  {
  float ITOF2;

  ITOF2 = ITOF * ITOF;

  SllWetted =   ITOF2 * SllWetted;
  Sll = ITOF2 * Sll;

  LugLength = ITOF * LugLength;
  }

{
float CfBody;
CfBody = GetCfBody (Velocity, LugLength, Surface);
CfBody = CfBody;
}

/* From Estes TR11 P 26. Note that .0045 is a Cf estimate in the formula */
/* Units are in feet because of GetCfBody call - Compare to Gregorek's Inches */

CdLug = (1.2 * Sll + GetCfBody (Velocity, LugLength, Surface) * SllWetted)
		/GlobalXSection;

return(CdLug);
}


/****************************************************************************/
float GetSectionLugCd(SECTIONTYPE * SectionPointer, float Velocity)
{
LUGTYPE * LugPointer;
int i;
float LugCd = 0.0;

LugPointer = SectionPointer->Lugs;

   for(i=0; i < SectionPointer->NumLugs; i++)
   {

       LugCd +=GetLugCd(
		       LugPointer-> Sll ,
		       LugPointer->SllWetted,
		       LugPointer->Length,
		       LugPointer->Surface,
		       Velocity
		       );
	LugPointer = LugPointer->Next;
   }

return(LugCd);
}
/****************************************************************************/
float GetSectionFinCd(SECTIONTYPE * SectionPointer, float Velocity)
{
FINTYPE * FinPointer;
int i;
float FinCd = 0.0;

FinPointer = SectionPointer->Fins;

   for(i=0; i < SectionPointer->NumFinSets; i++)
   {

       FinCd += GetFinCd(Velocity,
			 FinPointer->AverageChord,
			 FinPointer->Thickness,
			 FinPointer->Surface,
			 FinPointer->Width,
			 (float) FinPointer->NumFins,
			 FinPointer->BodyDiameter,
			 FinPointer->RootChord,
			 FinPointer->TipChord,
			 FinPointer->TipType,
			 FinPointer->DoExtended,
			 FinPointer->FinType
			 );

	FinPointer = (FINTYPE *) FinPointer->Next;
   }

return(FinCd);
}
/****************************************************************************/
float GetStageLugCd(STAGETYPE * StagePointer, float Velocity)
{
int i;
SECTIONTYPE * Pointer;
float LugCd = 0.0;

Pointer = StagePointer->Sections;

for (i = 0; i< StagePointer->NumSections; i++)
   {
   LugCd += GetSectionLugCd(Pointer, Velocity);
   Pointer = (SECTIONTYPE *) Pointer->Next;
   }
return(LugCd);
}
/****************************************************************************/
float GetStageFinCd(STAGETYPE * StagePointer, float Velocity)
{
int i;
SECTIONTYPE * Pointer;
float FinCd = 0.0;

Pointer = StagePointer->Sections;

for (i = 0; i< StagePointer->NumSections; i++)
   {
   FinCd += GetSectionFinCd(Pointer, Velocity);
   Pointer = (SECTIONTYPE *) Pointer->Next;
   }
return(FinCd);
}
/****************************************************************************/
float GetCd(CONFIGTYPE * Rocket, float Velocity)
{
int i;
float BaseCd, Cd;
STAGETYPE * Pointer;

Pointer = Rocket->Stages;

GlobalFinCd = GlobalLugCd = 0.0;

if(Velocity == (float) 0.0) Velocity = .01;

for (i=0; i < Rocket->NumStages; i++)
    {
    GlobalFinCd += GetStageFinCd(Pointer, Velocity);
    GlobalLugCd += GetStageLugCd(Pointer, Velocity);
    Pointer = (STAGETYPE *) Pointer->Next;
    }



GlobalNoseBodyCd = GetNoseBodyCd(Velocity,
				 Rocket->TotalLength,
				 Rocket->MaximumDiameter,
				 Rocket->BaseDiameter,
				 Rocket->Surface,
				 Rocket->TotalWettedAreaOfBody
				 );


return(GlobalFinCd + GlobalNoseBodyCd + GlobalLugCd
);

}
/****************************************************************************/
void PrintOutput(FILE * Unit, CONFIGTYPE * Rocket)
{
float Force, Moment, Cd;
char LineOut[80];



PrintCenter(Unit, Title);
sprintf(LineOut,"(Temperature is %f degrees %s)\n\n",
 Temperature1, TempUnitsDollar[TempUnits]);

PrintCenter(Unit, LineOut);

if (Units != METRIC)
   {
   Velocity = 100.0;
   Cd = GetCd(Rocket, Velocity);

   fprintf(Unit, "Cd at 100 ft/sec = %f\n", Cd);
   fprintf(Unit, "Nose-BodyCd= %f (BaseCd= %f), Fin Cd= %f, Lug Cd = %f\n\n\n",
	   GlobalNoseBodyCd, GlobalBaseCd, GlobalFinCd, GlobalLugCd);

   Velocity = 150.0;
   Cd = GetCd(Rocket, Velocity);
   fprintf(Unit, "Cd at 150 ft/sec = %f\n", Cd);
   fprintf(Unit, "Nose-BodyCd= %f (BaseCd= %f), Fin Cd= %f, Lug Cd = %f\n\n\n",
	   GlobalNoseBodyCd, GlobalBaseCd, GlobalFinCd, GlobalLugCd);

   Velocity = 200.0;
   Cd = GetCd(Rocket, Velocity);
   fprintf(Unit, "Cd at 200 ft/sec = %f\n", Cd);
   fprintf(Unit, "Nose-BodyCd= %f (BaseCd= %f), Fin Cd= %f, Lug Cd = %f\n\n\n",
	   GlobalNoseBodyCd, GlobalBaseCd, GlobalFinCd, GlobalLugCd);


   Velocity = 250.0;
   Cd = GetCd(Rocket, Velocity);
   fprintf(Unit, "Cd at 250 ft/sec = %f\n", Cd);
   fprintf(Unit, "Nose-BodyCd= %f (BaseCd= %f), Fin Cd= %f, Lug Cd = %f\n\n\n",
	   GlobalNoseBodyCd, GlobalBaseCd, GlobalFinCd, GlobalLugCd);

   fprintf(Unit, "\nOverall CP = %f Inches From Nose Tip\n\n", GlobalCP);
   }
 else
   {
   Velocity = 25.0 * MTOF;
   Cd = GetCd(Rocket, Velocity);
   fprintf(Unit, "Cd at  25 m/sec = %f\n", Cd);
   fprintf(Unit, "Nose-BodyCd= %f (BaseCd= %f), Fin Cd= %f, Lug Cd = %f\n\n\n",
	   GlobalNoseBodyCd, GlobalBaseCd, GlobalFinCd, GlobalLugCd);

   Velocity = 50.0 * MTOF;
   Cd = GetCd(Rocket, Velocity);
   fprintf(Unit, "Cd at  50 m/sec = %f\n", Cd);
   fprintf(Unit, "Nose-BodyCd= %f (BaseCd= %f), Fin Cd= %f, Lug Cd = %f\n\n\n",
	   GlobalNoseBodyCd, GlobalBaseCd, GlobalFinCd, GlobalLugCd);

   Velocity = 75.0 * MTOF;
   Cd = GetCd(Rocket, Velocity);
   fprintf(Unit, "Cd at  75 m/sec = %f\n", Cd);
   fprintf(Unit, "Nose-BodyCd= %f (BaseCd= %f), Fin Cd= %f, Lug Cd = %f\n\n\n",
	   GlobalNoseBodyCd, GlobalBaseCd, GlobalFinCd, GlobalLugCd);


   Velocity = 100.0 * MTOF;
   Cd = GetCd(Rocket, Velocity);
   fprintf(Unit, "Cd at 100 m/sec = %f\n", Cd);
   fprintf(Unit, "Nose-BodyCd= %f (BaseCd= %f), Fin Cd= %f, Lug Cd = %f\n\n\n",
	   GlobalNoseBodyCd, GlobalBaseCd, GlobalFinCd, GlobalLugCd);

   fprintf(Unit, "\nOverall CP = %f Centimeters From Nose Tip\n\n", GlobalCP);
   }

fflush(Unit);
return;
}
/****************************************************************************/
void PrintConfig(CONFIGTYPE * Rocket)
{
#define SCREEN 1
#define DISK 2


int Destination = 0, WhileSw = TRUE;
int i, j, NumSections, PageSwitch=1;
FILE * OutFile;
char FileName[80]="", Answer[80];


char FF =12;
float x;

SECTIONTYPE * SectionPointer;
LUGTYPE * LugPointer;
FINTYPE * FinPointer;

NumSections = Rocket->Stages->NumSections;

CLS;

while((x = prompt("Destination (1=Screen, 2=Disk): ", Destination )) !=1
	&& x != 2);

Destination = (int) x;

if(Destination == 1)
  {
  CLS;
  OutFile=stdout;
  }
else
  {
	while(WhileSw == TRUE)
	   {
	   CLS;
	   PrintCenter(stdout, Title);
	   printf("\n\n Print File Name: ");
	   get_line(FALSE, FileName);
	   printf("Are You Sure? (y/n/q)");
	   get_line(TRUE, Answer);
	   if(!strcmp(Answer,"Y")) WhileSw = FALSE;
	   else if(!strcmp(Answer,"Q")) WhileSw = FALSE;
	   else printf("\n");
	   }

  if(!strcmp(Answer,"Q"))return;

  OutFile = fopen(FileName, "w");
  }

PrintCenter(OutFile, Title);

sprintf(LineOut,"(Temperature is %f degrees %s, Input Units are in %s)\n\n",
 Temperature1, TempUnitsDollar[TempUnits], InputUnits[Units]);

PrintCenter(OutFile, LineOut);

fprintf(OutFile, "Total Length = %f, Max Diameter = %f, # of Sections = %i\n",
	Rocket->TotalLength, Rocket->MaximumDiameter, NumSections);

fprintf(OutFile, "Nose Type= %s, %s Nose Length = %f, NoseDiameter = %f\n",
	NoseType[Rocket->NoseType], SharpOrBlunt[Rocket->SharpOrBluntNose],
	Rocket->NoseLength, Rocket->NoseDiameter);

 if(Rocket->BoatTailLength != 0.0)
   {
   fprintf(OutFile, "Boat Tail Length = %f, Base Diameter = %f\n",
	   Rocket->BoatTailLength, Rocket->BaseDiameter);
   }

SectionPointer = (SECTIONTYPE *) Rocket->Stages->Sections;

for(i=0; i < NumSections; i++)
   {
   fprintf(OutFile, "\n\nFor Section #%i:\n", NumSections - i);

   if(i != 0)
     {
     fprintf(OutFile, "Diameter = %f, Length = %f, Surface = %s \n",
     SectionPointer->Diameter, SectionPointer->Length,
     SurfaceType[Rocket->SurfaceType]);
     }
   else
     {
     fprintf(OutFile, "Diameter = %f, Length = %f, Surface = %s \n",
     SectionPointer->Diameter, SectionPointer->Length + Rocket->NoseLength,
     SurfaceType[Rocket->SurfaceType]);
     }

   if(SectionPointer->TransitionLength > 0.0)
     {
     fprintf(OutFile,"TransitionLength = %f, Transition Base = %f\n",
		      SectionPointer->TransitionLength,
		      SectionPointer->TransitionTipDiameter);
     }

     fprintf(OutFile,"Number of Lugs = %i", SectionPointer->NumLugs);

     LugPointer = (LUGTYPE *) SectionPointer->Lugs;
     for(j=0; j< SectionPointer->NumLugs; j++)
	{
	if(SectionPointer->NumLugs > 1)fprintf(OutFile, "\nFor Lug #%i:\n", j+1);
	fprintf(OutFile,"Lug OD = %f, Lug ID = %f, Lug Length = %f\n",
		LugPointer->OutsideDiameter, LugPointer->InsideDiameter,
		LugPointer->Length);

	LugPointer = (LUGTYPE *) LugPointer->Next;
	}

     fprintf(OutFile,"Number of Fin Sets for Section #%i is %i\n",
	     NumSections - i, SectionPointer->NumFinSets);

     FinPointer = (FINTYPE *) SectionPointer->Fins;

     for(j=0; j < SectionPointer->NumFinSets; j++)
	{
	if(SectionPointer->NumFinSets > 1)
	  {
	  fprintf(OutFile,"\nFor Fin Set #%i of Section #%i:\n",
	  j + 1, NumSections - i );
	  }
	else fprintf(OutFile,"\n");

	fprintf(OutFile,"%i Fins of Type %s, with %s Surface\n",
		FinPointer->NumFins, FinType[FinPointer->FinType],
		SurfaceType[FinPointer->SurfaceType]);

	fprintf(OutFile,
	"Root = %f, Tip = %f, Mean Chord = %f, Semi-Span = %f\n",
	 FinPointer->RootChord, FinPointer->TipChord,
	 FinPointer->AverageChord,FinPointer->Width);

	 fprintf(OutFile, "Thickness = %f, SweepBack = %f Dist From Nose = %f\n",
	  FinPointer->Thickness, FinPointer->Sweep, FinPointer->FromNose);

	 fprintf(OutFile, "Fin Tip Type = %s \n",TipType[FinPointer->TipType]);

	 FinPointer = (FINTYPE *) FinPointer->Next;
	}
   SectionPointer = (SECTIONTYPE *) SectionPointer->Next;
   if(Destination == SCREEN)
     {
     get_line(FALSE, Answer);
     CLS;
     }
   else /* Destination == DISK */
     {
     if((PageSwitch = 1-PageSwitch))
       {
       fprintf(OutFile,"%c", FF);
       }

     else fprintf(OutFile, "\n");

     } /* Else Destination == DISK */


   } /* For loop */

/*
if(Destination == SCREEN)
  {
  get_line(FALSE, Answer);
  CLS;
  }
else
  {
  if(Rocket->NumSections > 1)fprintf(OutFile,"%c", FF);
  else fprintf(OutFile, "\n");
  }
*/
PrintOutput(OutFile, Rocket);

if(Destination != SCREEN)fclose(OutFile);
return;
}
/***************************************************************************/

void GetTail(float * TailDiameter, float * TailLength, CONFIGTYPE * Rocket)
{
int i;
STAGETYPE * StagePointer;
SECTIONTYPE * SectionPointer;

StagePointer = Rocket->Stages;

/* Space over to Last (First) Stage. Start with 1 => Don't exceed */
for (i = 1; i < Rocket->NumStages; i++)
   {
   StagePointer = (STAGETYPE *) StagePointer->Next;
   }

SectionPointer = StagePointer->Sections;

for (i = 1; i < StagePointer->NumSections; i++)
   {
   SectionPointer = (SECTIONTYPE *) SectionPointer->Next;
   }

*TailDiameter = SectionPointer->TransitionTipDiameter;
*TailLength = SectionPointer->TransitionLength;

}
/****************************************************************************/
void NoseConeCP(float * CPLoc,
		float * COEFF,
		int NoseConeType,
		float NoseConeLength
		)

{

if(NoseConeLength == 0.0)
  {
  * COEFF = 0.0;
  * CPLoc = 0.0;
  return;
  }



* COEFF = 2.0;


switch(NoseConeType)
   {
   case OGIVE:   * CPLoc = 0.466 * NoseConeLength;
		 break;

   case CONE:    * CPLoc = 0.5 * NoseConeLength;
		 break;

   case ELLIPSE: * CPLoc = 0.666 * NoseConeLength;
		 break;

   default:      printf("INVALID NOSE TYPE %i\n", NoseConeType);
		 * CPLoc = 0.0;
   }

return;
}
/****************************************************************************/
void ConicalTransitionCP(
			float * CPLoc,
			float * COEFF,
			float TipDiam,
			float BaseDiam,
			float XLength,
			float FromNose,
			float NoseDiam
			)
{

if(XLength == 0)
  {
  * COEFF = 0.0;
  * CPLoc = 0.0;
  return;
  }


* COEFF = 2 * (( sqr(BaseDiam/NoseDiam) - sqr(TipDiam/ NoseDiam) ));
* CPLoc = FromNose + (XLength / 3) *(1.0 + (1.0 - (TipDiam / BaseDiam)) ) /
		       (1.0 - sqr(TipDiam / BaseDiam));
}
/****************************************************************************/
void GetTrapezoidalFinCP(
	       float * CPLoc,
	       float * COEFF,
	       int Number,
	       float Root,
	       float Tip,
	       float VFW,
	       float Sweep,
	       float FromNose,
	       float NoseDiam,
	       float BodyDiameter
	       )
{
float Radius, LF, FT1, FT2, FT3, FT4;

if((VFW == 0.0) || (Root == 0.0) || (Number == 0))
  {
  * COEFF = 0.0;
  * CPLoc = 0.0;
  return;
  }


Radius = BodyDiameter/2;

LF = sqrt(VFW * VFW + sqr(Sweep + (Tip - Root)/ 2) );

if(Number < 6) FT1 = 1.0 + (Radius / (VFW + Radius));
else FT1 = 1.0 + 0.5 * (Radius / (VFW + Radius)); /* Six Fins */


FT2 = 1.0 + sqrt(1 + sqr((2.0 * LF)/(Root + Tip)) );
* COEFF = FT1 *(( (float) (4 * Number) * sqr(VFW / NoseDiam)) / FT2);
FT3 = Root + Tip;
FT4 = (FT3 - (Root * Tip)/FT3)/6.0;

* CPLoc = FromNose + (Sweep / 3.0) *(FT3 + Tip)/FT3 + FT4;
return;
}
/****************************************************************************/
void GetEllipticalFinCP(
	       float * CPLoc,
	       float * COEFF,
	       int Number,
	       float Root,
	       float VFW,
	       float FromNose,
	       float NoseDiam,
	       float BodyDiameter
	       )

{
float LF, Radius, FT5;


if((VFW == 0.0) || (Root == 0.0) || (Number == 0))
  {
  * COEFF = 0.0;
  * CPLoc = 0.0;
  return;
  }

Radius = BodyDiameter/2;


FT5 = ( (float) (4 * Number) * sqr(VFW / NoseDiam)) /
	(1.0 + sqrt(1.0 + (1.623 * sqr(VFW / Root)) ));

if(Number < 6) * COEFF = FT5 * (1.0 + Radius/(VFW + Radius));
else * COEFF = FT5 * (1.0 + 0.5 * Radius/(VFW + Radius));

* CPLoc = FromNose + .288 * Root;

return;
}
/****************************************************************************/
void GetSectionCP(float * Force, float * Moment, SECTIONTYPE * SectionPointer)
{
int i;
float TotalMoment = 0.0, TotalForce = 0.0, CPLoc, COEFF;
FINTYPE * FinPointer;

FinPointer = SectionPointer->Fins;
for(i = 0; i < SectionPointer->NumFinSets; i++)
   {
   if(FinPointer->FinType == TRAPEZOIDAL)
     {
     GetTrapezoidalFinCP(
		       &CPLoc,
		       &COEFF,
		       FinPointer->NumFins,
		       FinPointer->RootChord,
		       FinPointer->TipChord,
		       FinPointer->Width,
		       FinPointer->Sweep,
		       FinPointer->FromNose,
		       Rocket->NoseDiameter, /* Global */
		       FinPointer->BodyDiameter
		       );
     }
   else
     {
     GetEllipticalFinCP(
		       & CPLoc,
		       & COEFF,
		       FinPointer->NumFins,
		       FinPointer->RootChord,
		       FinPointer->Width,
		       FinPointer->FromNose,
		       Rocket->NoseDiameter, /* Global */
		       FinPointer->BodyDiameter
		       );
      }/* if */

   TotalForce += COEFF;
   TotalMoment += CPLoc * COEFF;
   }/* for loop */

   if(SectionPointer->TransitionLength != 0.0)
     {
     ConicalTransitionCP(
			&  CPLoc,
			& COEFF,
			SectionPointer->Diameter,
			SectionPointer->TransitionTipDiameter,
			SectionPointer->TransitionLength,
			SectionPointer->FromNose, /* Calculate */
			Rocket->NoseDiameter
			);

     TotalForce += COEFF;
     TotalMoment += CPLoc * COEFF;
     }

* Moment = TotalMoment;
* Force = TotalForce;

return;
}
/****************************************************************************/
GetStageCP(float * Force, float * Moment, STAGETYPE * StagePointer)
{
int i;
SECTIONTYPE * SectionPointer;
float TotalMoment = 0, TotalForce = 0;

SectionPointer = StagePointer->Sections;
for(i=0; i< StagePointer->NumSections; i++)
   {
   GetSectionCP(Force, Moment, SectionPointer);
   TotalMoment += * Moment;
   TotalForce  += * Force;
   SectionPointer = (SECTIONTYPE *) SectionPointer->Next;
   }

* Moment = TotalMoment;
* Force = TotalForce;

return;

}
/****************************************************************************/
void GetCP(float * FinalForce, float * FinalMoment, float * FinalCP, CONFIGTYPE * Rocket)
{
float TotalForce = 0.0, TotalMoment = 0.0;
float Force, Moment, CPLoc, COEFF;
int i;
STAGETYPE * StagePointer;

NoseConeCP(& CPLoc,
	   & COEFF,
	   Rocket->NoseType,
	   Rocket->NoseLength
	  );

TotalForce = COEFF;
TotalMoment = CPLoc * COEFF;

StagePointer =  Rocket->Stages;
for(i = 0; i< Rocket->NumStages; i++)
   {
   GetStageCP(&Force, &Moment, StagePointer);
   StagePointer = (STAGETYPE *) StagePointer->Next;
   TotalMoment += Moment;
   TotalForce += Force;
   }

* FinalMoment = TotalMoment;
* FinalCP = TotalMoment / TotalForce;
* FinalForce = TotalForce;
return;

}

/****************************************************************************/

main()
{
int i, x, ContSwitch = TRUE, EntrySwitch;
float StageCd[100], WettedAreaOfBody, MaximumDiameter, TotalLength;
float * CD, CdMCrit;
char Label[80], Title1[80];
float Moment, Force, CP;

STAGETYPE * Pointer;


CLS;

#if TURBOWIN == 1
_InitEasyWin();
#endif

color_routine();

CLS;

Banner();

Title[0]='\0';

CLS;

if((CD = (float *) calloc(sizeof(float), DRAG_SIZE)) == (float *) NULL)
  {
  printf("Cannot Allocate Space For Drag Profile! Aborting !\n");
  exit(-99);
  }


while(((x = (int) prompt("0 = English,  1=Metric Input: ", (float) Units)) < 0)
       || (x > 1));
Units = x;


while(((x = (int) prompt("0 = Centigrade,  1=Fahrenheit: ", (float) TempUnits)) != 0.0)
	&&(x != 1));
TempUnits = x;

if(TempUnits == FAHRENHEIT) Temperature1 = TEMPF1;
else Temperature1 = TEMPC1;




if (Rocket == (CONFIGTYPE *) NULL)
   {
   if((Rocket = (CONFIGTYPE *) malloc(sizeof(CONFIGTYPE))) == (CONFIGTYPE *) NULL)
     {
     printf("\n** SEVERE ERROR!! **\nCANNOT ALLOCATE CONFIGURATION!\nABORTING\n");
     exit(-99);
     }
   memset(Rocket, 0, sizeof(CONFIGTYPE));
   Rocket->NoseType = 1; /* Ogive */
   Rocket->SurfaceType = 2; /* Smooth */
   Rocket->NumStages = 1;
   Rocket->Stages = (STAGETYPE *) NULL;
   Rocket->SharpOrBluntNose = SHARP;
   }/* IF NULL */

/* Note: Stage Structure is Top to Bottom as Linked List Progresses */

while(ContSwitch)
   {

   EntrySwitch = TRUE;

   while (EntrySwitch)
      {
      CLS;
      PrintCenter(stdout, Title);
      printf("Title: ");
      get_line(FALSE,Title1);
      if(Title1[0] != '\0')strcpy(Title, Title1);
      CLS;
      PrintCenter(stdout, Title);

      InputConfig(Rocket);

		  /** VISCOSITY REGRESSED FROM DATA **/
	/** SOURCE: SCHAUM'S OUTLINE FLUID MECHANICS & HYDRODYNAMICS P 246 **/

       GlobalKVisc = ((0.96204E-6 *(TemperatureK) -120.138E-6));

      /* Regression Cenetered around KVisc = 1/6100 at 20 degrees C */
     /* GlobalKVisc = 1.0/6100.0 + .98032e-6 *(TemperatureK -293.17); */


      Mach1 = get_mach1(TemperatureK);

      /* Could do these in one routine, but they're short. Let's be clear */
      Rocket->MaximumDiameter = GetRocketDiameter(Rocket);

      /* Get Global XSectional Area */
      GlobalXSection = PI * Rocket->MaximumDiameter *
			    Rocket->MaximumDiameter / 4.0;

      /* Global Variables Must Be in End Units */
      if(Units == METRIC) GlobalXSection *= CTOF * CTOF;
      else GlobalXSection *= ITOF * ITOF;


      Rocket->TotalLength = GetTotalLength(Rocket);

      Rocket->TotalWettedAreaOfBody = GetWettedAreaOfBody(Rocket);

      GetTail(&(Rocket->BaseDiameter), &(Rocket->BoatTailLength), Rocket);

      /*@@@@@@@@@@@@ Prompts here to generate Cd's or what-you-like */

      printf("\n");
      while(((x = (int)  prompt("0 = Comput Cd's   1= Re-Enter: ", (float) EntrySwitch)) != 0.0)
	   &&(x != 1));
      EntrySwitch= x;
      }/* While EntrySwitch */

/*====================  Calculations Executive Routine  ===================*/


IMach1 = (int) Mach1;

#if MCRIT == 8
IMCrit = (int) ((float) 0.8 * IMach1);
#else
IMCrit = (int)((float) 0.9 * IMach1);
#endif
CLS;


GetCP(&Force, &Moment, &GlobalCP, Rocket);

PrintOutput(stdout, Rocket);

/*==================  End Calculations Executive Routine  =================*/
   while(((x =  (int) prompt("0 = Quit   1= Another Run   2 = Save Profile 3 = Print Data: ",
    (float) ContSwitch)) != 0.0) && (x != 1.0))
      {
      if (x == 3) PrintConfig(Rocket);
      else if(x == 2)
	{
	FILE * fd;
	char FileName[LINE_LENGTH], Answer[5];
	int WhileSw = TRUE;
	while(WhileSw == TRUE)
	   {
	   CLS;
	   PrintCenter(stdout, Title);
	   printf("\n\n Output File Name: ");
	   get_line(FALSE, FileName);
	   printf("Are You Sure? (y/n/q)");
	   get_line(TRUE, Answer);
	   if(!strcmp(Answer,"Y")) WhileSw = FALSE;
	   else if(!strcmp(Answer,"Q")) WhileSw = FALSE;
	   else printf("\n");
	   }

	 if(strcmp(Answer,"Q"))
	   {
	   fd = fopen(FileName, "w");
	   /* OutPut Title */
	   fprintf(fd, "#%s\n", Title);
       if(Units == METRIC)
	     {
	     fprintf(fd, "#Center of Pressure in In From Nose Tip = %f\n", GlobalCP);
	     }
	   else
	     {
	     fprintf(fd, "#Center of Pressure in Cm From Nose Tip = %f\n", GlobalCP);
	     }

	   for(i = 0, Velocity = 0.0; i < DRAG_SIZE; i++, Velocity += MTOF)
	      {
	      CD[i] = GetCd(Rocket, Velocity);
	      }
   /* Value large & Meaningless at V=0. Fudge below better for graphs. */
	   CD[0] = CD[1] + 0.25;

	   machmaker( CD, TemperatureK, (int) Rocket->SharpOrBluntNose);


	   for (i=2; i< NUMBER_COMMENTS; i++)
	       fprintf(fd, "#\n");


	       for(i=0; i< DRAG_SIZE; i++)
		  {
		  fprintf(fd, "%f\n", CD[i]);
		  }
	       fclose(fd);
	   }/* if(strcmp... */
	}
      }
   ContSwitch= x;
   }/* While ContSwitch */
}
